//---------------------------------------------------------------------------
#ifndef SkemaPlanH
#define SkemaPlanH
#include <cstring.h>
#include <conio.h>
#include <iostream.h>
#include <fstream.h>
//#include ".h"
#include <Printers.hpp>
#include <vcl/graphics.hpp>


//---------------------------------------------------------------------------

struct SkemaPlan
{
    SkemaPlan();
    void udskrivSkemaPlan(int type);
    bool SkemaTilFil(string FilNavn, string SkemaNavn, char valg);

    void SkemaPlan::Ny(string FilNavn);
    void SkemaTilPrinter(string SkemaNavn);
    bool IndsaetPaaPlan(string hold, string fag, string laerer,
							   string lokale, int col, int row);
    void NulstilTemp();

    char SPlan[28][85];
    char Temp[28][85];
};


#endif
